<?php

namespace App\Http\Controllers;

use App\Helper\Utility;
use Illuminate\Http\Request;
use App\Model\QuizModel;
use Illuminate\Support\Facades\Auth;
use Log;

class QuizController extends Controller
{

    /**
     * Quiz model instance for database operations.
     * 
     * @var QuizModel
     */
    private $quizModel;

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        date_default_timezone_set('America/Los_Angeles');
        $this->quizModel = new QuizModel();
    }

    /**
     * Gets the customer details from the Shopify API, then gets the session IDs
     * from the database, then gets the quiz details from the database
     * 
     * Endpoint URL: https://<base_url>/getQuizDetails
     * 
     * @param Request $request The request object contain the customer `id`.
     * 
     * @return view There are two views that can be returned: `customerQuiz` and `error`.
     * `customerQuiz` the response object will be structured as follows.
     * {
     *   "customerName": "Rob",
     *   "email": "testing@p80w.com",
     *   "quizCount": 1,
     *   "quizDetails": `Customer quiz details`,
     * }
     * `error` the response object will be structured as follows.
     * {
     *   "error": `Error message`,
     *   "message": [
     *    "id" => `Error message` // Error message for the `id` field (optional)
     *   ],
     * }
     */
    public function getQuizDetails(Request $request)
    {
        $customerID = $request->input('id') ?? '';
        if (empty($customerID)) {
            $error = [
                'error' => 'Missing required field',
                'message' => [
                    'id' => 'Customer Id is required',
                ]
            ];
            Log::channel('quizControllerLog')->error("getQuizDetails: Validation error: " . json_encode($error));
            return view('error', compact('error'));
        }
        try {
            $shop = Auth::user();

            $customerDetails = (object) $shop->api()->rest('GET', '/admin/customers/' . $customerID . '.json', []);

            $customerDetails = $customerDetails->body;
            $email = isset($customerDetails->customer->email) ? $customerDetails->customer->email : '';
            $firstName = isset($customerDetails->customer->first_name) ? $customerDetails->customer->first_name : '';
            $lastName = isset($customerDetails->customer->last_name) ? $customerDetails->customer->last_name : '';
            $customerName = $firstName . " " . $lastName;

            $sessionIDs = $this->quizModel->getSessionIds($email);

            if (!empty($sessionIDs)) {
                $quizDetails = $this->quizModel->getQuizDetails($sessionIDs);
            }
            return view(
                'customerQuiz',
                [
                    "customerName" => $customerName,
                    "email" => $email,
                    "quizCount" => isset($quizDetails) ? count($quizDetails) : 0,
                    "quizDetails" => isset($quizDetails) ? $quizDetails : '',
                    "sessionIds" => implode(",", $sessionIDs)
                ]
            );
        } catch (\Exception $e) {
            $error = [
                'error' => $e->getMessage(),
            ];
            Log::channel('quizControllerLog')->error("quizList: " . $e->getMessage());
            return view('error', compact('error'));
        }
    }

    /**
     * Gets the quiz details by the session ID and returns the view with the quiz details
     * 
     * Endpoint URL: https://<base_url>/quizData
     * 
     * @param Request $request The request object contains the `sessionId` and `email`.
     * 
     * @return view There are two views that can be returned: `quizDetails` and `error`.
     * `quizDetails` the response object will be structured as follows.
     * {
     *   "quizDetails": `Quiz details`,
     *   "sessionId": "abcd-1234-5678-9efg",
     *   "recommendations": `Recommendations`,
     *   "quizCompletionTime": [
     *     "min": 1,
     *     "sec": 30
     *   ],
     *   "quizTime": "2020-02-22T12:30:00-08:00",
     *   "email": "testing@p80w.com",
     *   "responseTime": "0.00"
     * }
     * `error` the response object will be structured as follows.
     * {
     *  "error": `error message`,
     *  "message": [
     *    "sessionId" => `Error message` // Error message for the `sessionId` field (optional)
     *  ],
     * }
     */
    public function getQuizDetailById(Request $request)
    {
        $sessionID = $request->input('sessionId') ?? '';
        $email = $request->input('email') ?? '';

        if (empty($sessionID)) {
            $error = [
                'error' => 'Missing required field',
                'message' => [
                    'sessionId' => 'Session Id is required',
                ]
            ];

            Log::channel('quizControllerLog')->error("getQuizDetailById: Validation error: " . json_encode($error));
            return view('error', compact('error'));
        }

        try {

            $quizDetails = $this->quizModel->getQuizDetails([$sessionID]);
            $recommendations = $this->quizModel->getRecommendation($sessionID);

            $updatedDateTime = $quizDetails[$sessionID][0]->created_at;

            $updatedTime = Utility::getQuizDate($updatedDateTime);

            $responseTime = $this->quizModel->getResponseTime($sessionID);

            return view(
                'quizDetails',
                [
                    "quizDetails" => $quizDetails[$sessionID],
                    "sessionId" => $sessionID,
                    "recommendation" => !empty($recommendations) ? json_decode($recommendations) : '',
                    "quizCompletionTime" => $this->getQuizCompletionTime($quizDetails[$sessionID]),
                    "quizTime" => $updatedTime,
                    "email" => $email,
                    "responseTime" => $responseTime ?: '0.00',
                    "sessionIds" => $sessionID
                ]
            );
        } catch (\Exception $e) {
            $error = [
                'error' => $e->getMessage(),
            ];
            Log::channel('quizControllerLog')->error("quizList: " . $e->getMessage());
            return view('error', compact('error'));
        }
    }

    /**
     * Gets the customer details from the database and displays them in a view
     * 
     * Endpoint URL: https://<base_url>/customerData
     * 
     * @param Request $request The request object contains the `email` and email.
     * 
     * @return view There are two views that can be returned: `customerQuiz` and `error`.
     * `customerQuiz` the response object will be structured as follows.
     * {
     *   "customerName": "Rob",
     *   "email": "testing@p80w.com",
     *   "quizCount": 1,
     *   "quizDetails": `Customer quiz details`,
     * }
     * `error` the response object will be structured as follows.
     * {
     *   "error": `Error message`,
     *   "message": [
     *    "id" => `Error message` // Error message for the `id` field (optional)
     *   ],
     * }
     */
    public function getCustomerDetails(Request $request)
    {
        $email = $request->input('email') ?? '';
        $customerName = $request->input('name') ?? '';

        if (empty($email)) {
            $error = [
                'error' => 'Missing required field',
                'message' => [
                    'email' => 'Email is required',
                ]
            ];

            Log::channel('quizControllerLog')->error("getCustomerDetails: Validation error: " . json_encode($error));
            return view('error', compact('error'));
        }
        try {

            $sessionIDs = $this->quizModel->getSessionIds($email);

            if (!empty($sessionIDs)) {
                $quizDetails = $this->quizModel->getQuizDetails($sessionIDs);
            }
            return view(
                'customerQuiz',
                [
                    "customerName" => $customerName,
                    "email" => $email,
                    "quizCount" => isset($quizDetails) ? count($quizDetails) : 0,
                    "quizDetails" => isset($quizDetails) ? $quizDetails : '',
                    "sessionIds" => implode(",", $sessionIDs)
                ]
            );
        } catch (\Exception $e) {
            $error = [
                'error' => $e->getMessage(),
            ];
            Log::channel('quizControllerLog')->error("quizList: " . $e->getMessage());
            return view('error', compact('error'));
        }
    }

    /**
     * Gets the list of customers from the shopify API, gets the quiz details
     * from the database, and then returns the view with the data
     * 
     * Endpoint URL: https://<base_url>/
     * 
     * @param Request $request The request object contains the following properties shop, hmac and page.
     * 
     * @return view There are two views that can be returned: `quizList` and `error`.
     * `quizList` the response object will be structured as follows.
     * {
     *  "customers": `Customer details`,
     *  "quizDetails": `Quiz details`,
     *  "previousLink": `Previous page link`,
     *  "nextLink": `Next page link`,
     *  "percentDetails": `Percentage details`,
     *  "domain": `Shop domain`,
     *  "hmac": `Hmac`,
     * }
     * `error` the response object will be structured as follows.
     * {
     *   "error": `Error message`
     * }
     */
    public function quizList(Request $request)
    {
        $limit = 20;
        $previousLink = '';
        $nextLink = '';
        $getPageLink = '';
        $shop    = Auth::user();

        try {
            $domain = $request->input('shop');
            $hmac =  $request->input('hmac');

            if (!$request->has('page')) {
                $data = (object) $shop->api()->rest(
                    'GET',
                    '/admin/customers.json',
                    ["fields" => "email, first_name, last_name", "limit" => $limit]
                );
                $previousLink = !empty($data->link->previous) ? $data->link->previous : '';
                $nextLink = !empty($data->link->next) ? $data->link->next : '';
            } else {
                $getPageLink = trim($request->input('page'));
                $data = (object) $shop->api()->rest(
                    'GET',
                    '/admin/customers.json',
                    ["limit" => $limit, 'page_info' => $getPageLink]
                );

                $previousLink = !empty($data->link->previous) ? $data->link->previous : '';
                $nextLink = !empty($data->link->next) ? $data->link->next : '';
            }

            $customerData = [];
            $customerEmail = [];
            foreach ($data->body->customers as $customer) {
                $email = $customer->email;
                $name = $customer->first_name . " " . $customer->last_name;
                $temp = array("email" => $email, "name" => $name);
                $customerData[] = $temp;
                $customerEmail[] = $email;
            }

            $quizDetails = $this->quizModel->getQuizDetailsByEmail($customerEmail);

            $customerNames = $this->quizModel->getCustomerNamesByEmail($customerEmail);

            $percentDetails = $this->getPercentDetails($quizDetails);

            return view('welcome', [
                "customers" => $customerData,
                "quizDetails" => $quizDetails,
                "previousLink" => $previousLink,
                "nextLink" => $nextLink,
                "percentDetails" => $percentDetails,
                "customerNames" => $customerNames,
                "domain" => $domain,
                "hmac" => $hmac
            ]);
        } catch (\Exception $e) {
            $error = [
                'error' => $e->getMessage(),
            ];
            Log::channel('quizControllerLog')->error("quizList: " . $e->getMessage());
            return view('error', compact('error'));
        }
    }

    /**
     * It takes the quiz details, and returns the difference between the first
     * question's completion_time and the last question's completion_time
     * 
     * @param quizDetails This is an array of objects that contains the details of
     * the quiz.
     */
    private function getQuizCompletionTime($quizDetails)
    {
        if (!empty($quizDetails)) {
            $startTime = new \DateTime($quizDetails[0]->completion_time);
            $endTime = new \DateTime(end($quizDetails)->completion_time);

            $completionTime = $startTime->diff($endTime);
            return array("min" => $completionTime->i, "sec" => $completionTime->s);
        }
        return null;
    }

    /**
     * It takes the quiz details and returns the quiz completion percentage of the quizzes attempted by the user
     * 
     * @param $quizDetails This is an array of all the quiz details for a particular
     * user.
     * 
     * @return array $percentDetails This is an array of the quiz completion percentage
     */
    private function getPercentDetails($quizDetails)
    {
        $percentDetails = array();
        if (!empty($quizDetails)) {
            foreach ($quizDetails as $email => $details) {
                $latestSessionID = end($details)['session_id'];
                $quizData = $this->quizModel->getQuizDetails([$latestSessionID]);
                $quizData = array_values($quizData);
                if ($quizData[0][0]->isrecommendation == 'Yes') {
                    $percentDetails[$email] = '100';
                } else {
                    $count = count($quizData[0]);
                    $percentDetails[$email] = ($count / 50) * 100;
                }
            }
        }
        return $percentDetails;
    }

    /**
     * Delete customer details and quiz history from DB
     * 
     * Endpoint URL: https://<base_url>/deleteQuiz
     * 
     * @param Request $request The request object contains the properties sessionIds and email.
     * 
     * The endpoint response will be true or false.
     * 
     * @return \Illuminate\Http\Response
     */
    public function deleteQuiz(Request $request) 
    {
        $quizSessionIds = [];
        $sessionIds = $request->input('sessionIds');
        $emailAddress = $request->input('email');
        if (empty($sessionIds)) {
            Log::channel('quizControllerLog')->info("SessionId is empty in the request, 
            sessionIds:".$sessionIds.", email:".$emailAddress);
        }
        $quizSessionIds = explode(",", $sessionIds);
        $deleteStatus = $this->quizModel->deleteQuiz($quizSessionIds, $emailAddress);
        return $deleteStatus;
    }
}
