<?php

namespace App\Model;

use App\Helper\Utility;
use DB;

class QuizModel
{
    /**
     * The database connection to use.
     * @var string
     */
    private $connection = 'mysql2';

    /**
     * Get quiz details based on the session IDs.
     * 
     * @param array $sessionIDs
     * 
     * @return array
     */
    public function getQuizDetails(array $sessionIDs)
    {
        $quizDetails = DB::connection($this->connection)
            ->table('user_responses as ur')
            ->selectRaw('
                ur.session_id,
                qu.created_at,
                qm.question,
                case when rec.session_id is null then "No" else "Yes" end as isrecommendation,
                IFNULL(qr.response,ur.response_text) as response,
                ur.created_at as completion_time,
                rec.first_name,
                rec.email
            ')
            ->join('question_master as qm', 'qm.id', '=', 'ur.qn_id')
            ->leftJoin('quiz_user as qu', 'qu.session_id', '=', 'ur.session_id')
            ->leftJoin('question_responses as qr', 'ur.response_id', '=', 'qr.id')
            ->leftJoin('recommendations as rec', 'ur.session_id', '=', 'rec.session_id')
            ->whereIn('ur.session_id', $sessionIDs)
            ->get();

        return $this->formatQuizDetails($quizDetails);
    }

    /**
     * Group the quiz details by session ID.
     * 
     * @param object $quizDetails
     * 
     * @return array
     */
    private function formatQuizDetails(object $quizDetails)
    {
        $quizData = [];
        if (!empty($quizDetails)) {
            foreach ($quizDetails as $data) {
                $data->created_at = Utility::getQuizDate($data->created_at);
                if (!isset($quizData[$data->session_id])) {
                    $quizData[$data->session_id][] = $data;
                } else {
                    $quizData[$data->session_id][] = $data;
                }
            }
        }

        return $quizData;
    }

    /**
     * Get the recommendation for a given session ID
     * 
     * @param string $sessionID
     * 
     * @return string|null
     */
    public function getRecommendation(string $sessionID)
    {
        if (empty($sessionID)) {
            return null;
        }
        $quizRecommendation = DB::connection($this->connection)
            ->table('recommendations')
            ->select('recommendation')
            ->where('session_id', $sessionID)
            ->first();

        return $quizRecommendation->recommendation ?? null;
    }

    /**
     * Get quiz details based on the email addresses.
     * 
     * @param array $emailIds
     * @return array
     */
    public function getQuizDetailsByEmail(array $emailIds)
    {
        $quizDetails = DB::connection($this->connection)
            ->table('quiz_user as qu')
            ->selectRaw('
                qu.session_id,
                qu.created_at,
                u.user_email,
                case when rec.session_id is null then "No" else "Yes" end as isrecommendation
            ')
            ->leftJoin('users as u', 'u.id', '=', 'qu.user_id')
            ->leftJoin(
                DB::raw('(SELECT DISTINCT session_id from recommendations) AS rec'),
                'qu.session_id',
                '=',
                'rec.session_id'
            )
            ->whereIn('u.user_email', $emailIds)
            ->get();
        if (empty($quizDetails)) {
            return [];
        }
        $quizResult = [];

        foreach ($quizDetails as $quizData) {

            $quizData->created_at = Utility::getQuizDate($quizData->created_at);

            if (!isset($quizResult[$quizData->user_email])) {
                $quizResult[$quizData->user_email][] = (array)$quizData;
            } else {
                $quizResult[$quizData->user_email][] = (array)$quizData;
            }
        }

        return $quizResult;
    }

    /**
     * Get the customer quiz responses of the given email addresses.
     * 
     * @param array $emailIds
     * 
     * @return array
     */
    public function getCustomerNamesByEmail(array $emailIds)
    {
        $quizDetails = DB::connection($this->connection)
            ->table('quiz_user as qu')
            ->select('u.user_email', 'ur.response_text as name')
            ->leftJoin('users as u', 'u.id', '=', 'qu.user_id')
            ->leftJoin('user_responses as ur', 'ur.session_id', '=', 'qu.session_id')
            ->where('ur.qn_id', 2)
            ->whereIn('u.user_email', $emailIds)
            ->get();

        if (empty($quizDetails)) {
            return [];
        }

        $result = [];
        foreach ($quizDetails as $quizData) {
            $result[$quizData->user_email] = $quizData->name;
        }
        return $result;
    }

    /**
     * Get Session Ids based on the email.
     * 
     * @param string $email
     * 
     * @return array
     */
    public function getSessionIds(string $email)
    {
        if (empty($email)) {
            return [];
        }

        return DB::connection($this->connection)
            ->table('quiz_user as qu')
            ->select('qu.session_id')
            ->join('users as u', 'qu.user_id', '=', 'u.id')
            ->where('u.user_email', $email)
            ->pluck('qu.session_id')
            ->toArray();
    }

    /**
     * Get the response time of a given session Id.
     * 
     * @param string $sessionID
     * 
     * @return null|float
     */
    public function getResponseTime(string $sessionID)
    {
        if (empty($sessionID)) {
            return null;
        }

        $responseTime = DB::connection($this->connection)
            ->table('recommendations')
            ->where('session_id', $sessionID)
            ->value('response_time');

        return empty($responseTime) ? null : round($responseTime, 5);
    }

    /**
     * Delete customer and quiz details from DB
     * 
     * @param array $sessionIDs
     * @param array $email
     * 
     * @return true|false
     */
    public function deleteQuiz(array $sessionIDs, string $email)
    {
        $deleteResponseById = DB::connection($this->connection)->statement("DELETE quiz_user, user_responses, 
        recommendations FROM quiz_user INNER JOIN user_responses ON quiz_user.session_id = user_responses.session_id 
        INNER JOIN recommendations ON user_responses.session_id = recommendations.session_id WHERE quiz_user.session_id 
        IN (:sessionIds)", array("sessionIds" => implode(',',$sessionIDs)));
        
        $deleteResponseByEmail = DB::connection($this->connection)->table('users')->where('user_email', '=', $email)->delete();

        if ($deleteResponseById && $deleteResponseByEmail) {
            return true;
        }
        return false;
    }
}
